from django.conf import settings
AUTHENTICATION = getattr(settings, 'AUTHENTICATION', {})
AUTHENTICATION['REQUIRE_ACTIVATION'] = AUTHENTICATION.get('REQUIRE_ACTIVATION', True)
AUTHENTICATION['RESET_PASSWORD_EXPIRES'] = AUTHENTICATION.get('RESET_PASSWORD_EXPIRES', 20)

# TODO: Implement
# AUTHENTICATION['OPEN_REGISTRATION'] = AUTHENTICATION.get('OPEN_REGISTRATION', True)
# AUTHENTICATION['USERNAMES'] = AUTHENTICATION.get('USERNAMES', False)
