# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-12-04 11:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Contact',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('progenitor', models.CharField(blank=True, max_length=64, null=True)),
                ('type', models.CharField(blank=True, choices=[('', '-- select --'), ('Telephone', 'Telephone'), ('Email', 'Email'), ('Chat', 'Chat'), ('Profile', 'Profile'), ('Website', 'Website')], max_length=16, null=True)),
                ('value', models.CharField(blank=True, max_length=255, null=True)),
                ('label', models.CharField(blank=True, max_length=32, null=True)),
            ],
        ),
    ]
